@@ -34,5 +34,26 @@ else
 fi
 
 
+echo "#include <lo/lo.h>" >temp$$.c
+echo "main(){return 0;}" >>temp$$.c
+echo >>temp$$.c
+if pkg-config --cflags --libs liblo >/dev/null 2>/dev/null && gcc temp$$.c `pkg-config --cflags --libs liblo` 2>/dev/null ; then
+    echo "#define HAVE_LIBLO 1"
+    echo "//COMPILEFLAGS " `pkg-config --cflags --libs liblo`
+else
+    echo "#define HAVE_LIBLO 0"
+fi
+
+
+echo "#include <jack/jack.h>" >temp$$.c
+echo "main(){return (int)jack_port_get_latency_range;}" >>temp$$.c
+echo >>temp$$.c
+if gcc temp$$.c -ljack 2>/dev/null ; then
+    echo "#define NEW_JACK_LATENCY_API 1"
+else
+    echo "#define NEW_JACK_LATENCY_API 0"
+fi
+
+
 rm temp$$.c
 
